/*

	------------------------------------------------------------------------
	PNG-SurfaceProvider Class
	written by Karsten-Olaf Laux

	in order to works with
	ClanLib, the platform independent game SDK.

	This file is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		PNG (.png) surface provider.
*/

//! component="SurfaceProviders"

#ifndef header_PNGprovider
#define header_PNGprovider

#include "../Core/Display/pixelformat.h"
#include "../Core/SurfaceProviders/generic_surfaceprovider.h"
#include "../Core/SurfaceProviders/generic_surfaceprovider.h"
#include "../Core/System/clanstring.h"
#include "../Core/System/cl_assert.h"
#include "../Core/IOData/inputsource.h"
#include "../Core/IOData/inputsource_provider.h"

#include <png.h>

class CL_PNGProvider : public CL_SurfaceProvider_Generic
//: Surface provider that can load PNG (.png) files.
{
public:
	CL_InputSource* get_input_source() {return input_source; };
	//:returns Pointer to CL_InputSource
	//:(used by libpng-callback CL_PNGProvider::pngread_file() )

	static CL_Surface* create(CL_String handle, 
				  CL_InputSourceProvider* provider=NULL, 
				  bool transparent=true,
				  bool ignore_alphachannel=false);

	//: Loads the PNG file 'handle' from the inputsource provider 'provider. 
	//: Creates a CL_Surface using the PNG image and returns it.

	CL_PNGProvider(CL_String name, 
			 CL_InputSourceProvider *provider = NULL,
			 bool transparent=true,
			 bool ignore_alphachannel=false);
	//: Constructs a surface provider that can read PNG files.
	//!param: name - Name of the PNG file to load.
	//!param: provider - Input source provider that delivers the PNG file.
	//!param: transparent - True if a transparency color should be used.
	//!param: ignore_alphachannel - True if the alpha component should be ignored.
	//!param: trans_red - Red component of the transparency color.
	//!param: trans_green - Green component of the transparency color.
	//!param: trans_blue - blue component of the transparency color.

	virtual ~CL_PNGProvider();

	virtual unsigned int get_pitch() const { return pitch; }
	//: Returns the pitch of the image (bytes per line).

	virtual unsigned int get_width() const { return width; }
	//: Returns the width of the image.

	virtual unsigned int get_height() const { return height; }
	//: Returns the height of the image.
	
	virtual unsigned int get_num_frames() const { return 1; }
	//: Returns the number of subsprites in the image.

	virtual EPixelFormat get_pixel_format() const { return pixel_format; }
	//: Returns the pixelformat used by the image.

	virtual CL_Palette *get_palette() const { return palette; }
	//: Returns the palette used by the image. NULL if system palette.

	virtual unsigned int get_src_colorkey() const { return trans_col; }
	//: Returns the transparency color used.

	virtual bool uses_src_colorkey() const { return m_uses_src_colorkey; }
	//: Returns whether a source colorkey is used.

	virtual bool is_indexed() const { return true; }
	//: Returns whether the target uses an indexed color mode or not.
	
	virtual unsigned int get_red_mask() const;
	//: Returns the red color mask used by the target.

	virtual unsigned int get_green_mask() const;
	//: Returns the green color mask by the target.

	virtual unsigned int get_blue_mask() const;
	//: Returns the blue color mask by the target.

	virtual unsigned int get_alpha_mask() const;
	//: Returns the alpha mask by the target.

	virtual unsigned int CL_PNGProvider::get_depth() const;
	//: Returns the bit depth of the surface provider

	virtual void* get_data() const;
	//: Returns the image data. Provider must be locked before pointer is valid.

	virtual void perform_lock();
	//: Locks the surface provider.

	virtual void perform_unlock();
	//: Unlocks the surface provider.
	
	static void pngread_file(png_structp png_ptr,
				 png_bytep data, 
				 png_size_t length)
	//: Callback used by libpng to retrieve the filedata. 
	//: (calls get_input_source()->read_uchar8())
	  {
	    //since this method is static, we need to know who we are ...
	    CL_PNGProvider *instance =  (CL_PNGProvider *)png_get_io_ptr(png_ptr);
	    //no error-checking here ....
	    /*png_size_t i;
	    for(i=0; i< length; i++)
	    data[i]=instance->get_input_source()->read_uchar8();*/
	    unsigned int read_length = instance->get_input_source ()->read(data, length);
	    cl_assert (read_length == length);
	  }

private:
	CL_String filename;
	int locked;

	bool m_uses_src_colorkey;
	
	unsigned char *image;
	int pitch;
	int no_sprs;
	int width;
	int height;

	EPixelFormat pixel_format;

	bool transparent;
	bool ignore_alphachannel;

	CL_Palette* palette;
	bool indexed;
	int trans_col;
	int color_type;
	unsigned char trans_redcol, trans_greencol, trans_bluecol;

	void read_data();
	void read_data_rgb();
	void read_data_rgba();
	void read_data_grayscale();
	void read_data_grayscale_alpha();
	void read_data_palette();

	CL_InputSourceProvider *provider;
	CL_InputSource *input_source;

	//PNGlib stuff:
	png_structp png_ptr;
	png_infop info_ptr;
	png_infop end_info;

};

class CL_SetupPNG
{
public:
	static void init();
	static void deinit();
};

#endif

